#include "stdafx.h"
#include "clf_util.h"
#include "clf_resources.h"
#include "clf_shader.h"

static void printLog(char* prompt, GLuint obj)
{
	int infologLength = 0;
	int maxLength;

	if(glIsShader(obj))
		glGetShaderiv(obj,GL_INFO_LOG_LENGTH,&maxLength);
	else
		glGetProgramiv(obj,GL_INFO_LOG_LENGTH,&maxLength);

	char* infoLog = (char*)malloc(maxLength);

	if (glIsShader(obj))
		glGetShaderInfoLog(obj, maxLength, &infologLength, infoLog);
	else
		glGetProgramInfoLog(obj, maxLength, &infologLength, infoLog);

	if (infologLength > 0)
	{
		print(prompt);
		print("%s\n",infoLog);
	}
	free(infoLog);
}

clf_shader::clf_shader()
{
}

clf_shader::~clf_shader()
{
}

bool clf_shader::set_shaders(GLchar *vsSource, GLchar * fsSource)
{
	/* Compile and load the program */
	vs = glCreateShader(GL_VERTEX_SHADER);
	glShaderSource(vs, 1, (const GLchar**)&vsSource, NULL);
	glCompileShader(vs);
	int errc;
	glGetShaderiv(vs,GL_COMPILE_STATUS,&errc);
	if (errc == GL_FALSE)
	{
		print("vertex shader error");
		printLog("vertex shader", vs);
		return false;
	}

	fs = glCreateShader(GL_FRAGMENT_SHADER);
	glShaderSource(fs, 1, (const GLchar**)&fsSource, NULL);
	glCompileShader(fs);
	glGetShaderiv(fs,GL_COMPILE_STATUS,&errc);
	if (errc == GL_FALSE)
	{
		print("fragment shader error");
		printLog("fragment shader", fs);
		return false;
	}

	sp = glCreateProgram();
	glAttachShader(sp, vs);
	glAttachShader(sp, fs);
	glLinkProgram(sp);
	return true;
}

void clf_shader::terminate()
{
	glDeleteShader(vs);
	glDeleteShader(fs);
	glDeleteProgram(sp);
}

bool clf_shader::load_shaders(const char* vsh, const char* fsh)
{
	GLchar* vshader = load_file_buffer(get_resource_file_name (vsh,RES_SHD));
	GLchar* fshader = load_file_buffer(get_resource_file_name (fsh,RES_SHD));
	if (vshader && fshader)
		set_shaders(vshader, fshader);
	if (vshader) delete vshader;
	if (fshader) delete fshader;
	return true;
}

void clf_shader::begin()
{
	glUseProgram(sp);
}

void clf_shader::end()
{
	glUseProgram(0);
}
